/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ServerHealthModel.as 19387 2010-03-09 10:31:40Z mancelin $
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.admin.auditconsole.model {
import mx.collections.ArrayCollection;

import org.ow2.jonas.admin.auditconsole.util.IconsUtils;
import org.ow2.jonas.admin.auditconsole.vo.GenericAuditReportVO;
import org.ow2.jonas.admin.auditconsole.vo.InvocationAuditReportVO;
import org.ow2.jonas.admin.auditconsole.vo.JNDIAuditReportVO;
import org.ow2.jonas.admin.auditconsole.vo.WebInvocationAuditReportVO;

/**
 * Stats on the application server.
 */
[Bindable]
public class ServerHealthModel {

    [Embed("/../assets/blank.png")]
    private const blank:Class;

    private var totalTimeValue:Number = 0;
    private var totalCallsValue:Number = 0;
    private var totalTimeEJBValue:Number = 0;
    private var totalTimeWEBValue:Number = 0;
    private var numberEJBValue:Number = 0;
    private var numberWEBValue:Number = 0;
    private var numberJNDIValue:Number = 0;
    private var meanTimeValue:Number = 0;
    private var meanTimeEJBValue:Number = 0;
    private var meanTimeWEBValue:Number = 0;

    private var actualTrend:Number = 0;
    private var previousTrend:Number = 0;
    private var actualDurationTrend:Number = 0;
    private var previousDurationTrend:Number = 0;
    private var fiveMinutesTrend:Number = 0;
    private var tenMinutesTrend:Number = 0;
    private var fiveMinutesDurationTrend:Number = 0;
    private var tenMinutesDurationTrend:Number = 0;

    private var actualEJBTrend:Number = 0;
    private var previousEJBTrend:Number = 0;
    private var actualEJBDurationTrend:Number = 0;
    private var previousEJBDurationTrend:Number = 0;

    private var actualWEBTrend:Number = 0;
    private var previousWEBTrend:Number = 0;
    private var actualWEBDurationTrend:Number = 0;
    private var previousWEBDurationTrend:Number = 0;

    private var actualJNDITrend:Number = 0;
    private var previousJNDITrend:Number = 0;

    private var tendTimestamp:Number = 0;

    private var count:Number = 0;

    public var stats:ArrayCollection = new ArrayCollection();

    public var message:String = "";

    public function ServerHealthModel() {
        stats.addItem({name:"JOnAS requests average duration", data:"No data"});
        stats.addItem({name:"JOnAS total requests duration", data:"No data"});
        stats.addItem({name:"JOnAS requests count", data:"No data"});
        stats.addItem({name:"", data:""});
        stats.addItem({name:"EasyBeans requests average duration", data:"No data"});
        stats.addItem({name:"EasyBeans total request duration", data:"No data"});
        stats.addItem({name:"EasyBeans request count", data:"No data"});
        stats.addItem({name:"", data:""});
        stats.addItem({name:"Tomcat requests average duration", data:"No data"});
        stats.addItem({name:"Tomcat total request duration", data:"No data"});
        stats.addItem({name:"Tomcat request count", data:"No data"});
        stats.addItem({name:"", data:""});
        stats.addItem({name:"JNDI request count", data:"No data"});
    }

    public function addReport(report:GenericAuditReportVO):void {
        totalCallsValue++;
        actualTrend++;
        if (report is WebInvocationAuditReportVO) {
            var webReport:WebInvocationAuditReportVO = (report as WebInvocationAuditReportVO);
            totalTimeWEBValue += Number(webReport.requestDuration);
            totalTimeValue += Number(webReport.requestDuration);
            numberWEBValue++;
            actualWEBTrend++;
        }
        if (report is InvocationAuditReportVO) {
            var ejbReport:InvocationAuditReportVO = (report as InvocationAuditReportVO);
            totalTimeEJBValue += Number(ejbReport.requestDuration);
            totalTimeValue += Number(ejbReport.requestDuration);
            numberEJBValue++;
            actualEJBTrend++;
        }
        if (report is JNDIAuditReportVO) {
            numberJNDIValue++;
            actualJNDITrend++;
        }
        if (totalCallsValue > 0) {
            meanTimeValue = totalTimeValue / totalCallsValue;
        }
        if (numberEJBValue > 0) {
            meanTimeEJBValue = totalTimeEJBValue / numberEJBValue;
        }
        if (numberWEBValue > 0) {
            meanTimeWEBValue = totalTimeWEBValue / numberWEBValue;
        }
        var actualTimestamp:Number = new Date().getTime();
        if (actualTimestamp > (tendTimestamp + 30000)) {
            previousTrend = actualTrend;
            previousDurationTrend = actualDurationTrend;
            previousEJBTrend = actualEJBTrend;
            previousEJBDurationTrend = actualEJBDurationTrend;
            previousWEBTrend = actualWEBTrend;
            previousWEBDurationTrend = actualWEBDurationTrend;
            actualJNDITrend = previousJNDITrend;
            tendTimestamp = actualTimestamp;
            actualTrend = 0;
            actualEJBTrend = 0;
            actualWEBTrend = 0;
            actualJNDITrend = 0;
            count ++;
        }
        if (count == 10) {
            fiveMinutesTrend = actualTrend;
            fiveMinutesDurationTrend = actualDurationTrend;
            actualTrend = 0;
        }
        if (count == 20) {
            tenMinutesTrend = actualTrend;
            tenMinutesDurationTrend = actualDurationTrend;
            actualTrend = 0;
            count = 0;
        }
        actualDurationTrend = ((totalCallsValue * meanTimeValue) + report.duration) / (totalCallsValue + 1);
        actualEJBDurationTrend = ((totalTimeEJBValue * meanTimeEJBValue) + report.duration) / (totalTimeEJBValue + 1);
        actualWEBDurationTrend = ((totalTimeWEBValue * meanTimeWEBValue) + report.duration) / (totalTimeWEBValue + 1);
        updateStats();
    }

    public function updateStats():void {
        stats.removeAll();
        stats.addItem({
            name:"JOnAS requests average duration",
            data:meanTimeValue.toFixed(3) + " ms.",
            trend:getTrend(actualDurationTrend, previousDurationTrend)});
        stats.addItem({
            name:"JOnAS total requests duration",
            data:totalTimeValue.toFixed(3) + " ms.",
            trend:blank});
        stats.addItem({
            name:"JOnAS requests count",
            data:totalCallsValue,
            trend:getTrend(actualTrend, previousTrend)});
        stats.addItem({
            name:"",
            data:"",
            trend:blank});
        stats.addItem({
            name:"EasyBeans requests average duration",
            data:meanTimeEJBValue.toFixed(3) + " ms.",
            trend:getTrend(actualEJBDurationTrend, previousEJBDurationTrend)});
        stats.addItem({
            name:"EasyBeans total request duration",
            data:totalTimeEJBValue.toFixed(3) + " ms.",
            trend:blank});
        stats.addItem({
            name:"EasyBeans request count",
            data:numberEJBValue,
            trend:getTrend(actualEJBTrend, previousEJBTrend)});
        stats.addItem({
            name:"",
            data:"",
            trend:blank});
        stats.addItem({
            name:"Tomcat requests average duration",
            data:meanTimeWEBValue.toFixed(3) + " ms.",
            trend:getTrend(actualWEBDurationTrend, previousWEBDurationTrend)});
        stats.addItem({
            name:"Tomcat total request duration",
            data:totalTimeWEBValue.toFixed(3) + " ms.",
            trend:blank});
        stats.addItem({
            name:"Tomcat request count",
            data:numberWEBValue,
            trend:getTrend(actualWEBTrend, previousWEBTrend)});
        stats.addItem({
            name:"",
            data:"",
            trend:blank});
        stats.addItem({
            name:"JNDI request count",
            data:numberJNDIValue,
            trend:getTrend(actualJNDITrend, previousJNDITrend)});
    }
    
    private function getTrend(actual:int, previous:int):Class {
        var trend:Class = IconsUtils.getInstance().equal;
        if (actual < previous)
            trend = IconsUtils.getInstance().up;
        if (actual > previous)
            trend = IconsUtils.getInstance().down;
        return trend;
    }

    public function get totalTime():Number {
        return totalTimeValue;
    }

    public function set totalTime(value:Number):void {
        totalTimeValue = value;
    }

    public function get totalCalls():Number {
        return totalCallsValue;
    }

    public function set totalCalls(value:Number):void {
        totalCallsValue = value;
    }

    public function get totalTimeEJB():Number {
        return totalTimeEJBValue;
    }

    public function set totalTimeEJB(value:Number):void {
        totalTimeEJBValue = value;
    }

    public function get totalTimeWEB():Number {
        return totalTimeWEBValue;
    }

    public function set totalTimeWEB(value:Number):void {
        totalTimeWEBValue = value;
    }

    public function get numberEJB():Number {
        return numberEJBValue;
    }

    public function set numberEJB(value:Number):void {
        numberEJBValue = value;
    }

    public function get numberWEB():Number {
        return numberWEBValue;
    }

    public function set numberWEB(value:Number):void {
        numberWEBValue = value;
    }

    public function get numberJNDI():Number {
        return numberJNDIValue;
    }

    public function set numberJNDI(value:Number):void {
        numberJNDIValue = value;
    }

    public function get meanTime():Number {
        return meanTimeValue;
    }

    public function set meanTime(value:Number):void {
        meanTimeValue = value;
    }

    public function get meanTimeEJB():Number {
        return meanTimeEJBValue;
    }

    public function set meanTimeEJB(value:Number):void {
        meanTimeEJBValue = value;
    }

    public function get meanTimeWEB():Number {
        return meanTimeWEBValue;
    }

    public function set meanTimeWEB(value:Number):void {
        meanTimeWEBValue = value;
    }
}
}