/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: TopsAuditModel.as 19384 2010-03-08 18:41:14Z mancelin $
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.admin.auditconsole.model {
import mx.collections.ArrayCollection;

import mx.collections.Sort;
import mx.controls.Alert;

import org.ow2.jonas.admin.auditconsole.vo.GenericAuditReportVO;
import org.ow2.jonas.admin.auditconsole.vo.InvocationAuditReportVO;
import org.ow2.jonas.admin.auditconsole.vo.JNDIAuditReportVO;
import org.ow2.jonas.admin.auditconsole.vo.WebInvocationAuditReportVO;

/**
 * Most called resources on the application server.
 */
public class TopsAuditModel {

    private var topEjbResName:ArrayCollection = new ArrayCollection();

    private var topWebResName:ArrayCollection = new ArrayCollection();

    private var topJndiResName:ArrayCollection = new ArrayCollection();

    [Bindable]
    public var topJNDI:ArrayCollection = new ArrayCollection();

    [Bindable]
    public var topEJB:ArrayCollection = new ArrayCollection();

    [Bindable]
    public var topWEB:ArrayCollection = new ArrayCollection();

    public var topLimit:int = 5;

    public function TopsAuditModel() {}

    private function sizeCompare(a:Object, b:Object, fields:Array = null):int  {
        if (a == null && b == null) {
            return 0;
        }
        if (a == null) {
          return 1;
        }
        if (b == null) {
           return -1;
        }
        if (a.count < b.count) {
            return 1;
        }
        if (a.count > b.count) {
            return -1;
        }
        return 0;
    }

    private function incrementResource(key:String, duration:Number, collec:ArrayCollection):void {
        var found:Boolean = false;
        for each (var o:Object in collec) {
            if (o.name == key) {
                o.count = o.count + 1;
                o.duration = o.duration + duration;
                found = true;
                break;
            }
        }
        if (!found) {
            collec.addItem({name:key, count:1, duration:duration})
        }
        var sort:Sort = new Sort();
        sort.compareFunction = sizeCompare;
        collec.sort = sort;
        collec.refresh();
    }
    
    public function addReport(report:GenericAuditReportVO):void {
        if(report.isEJBInvocation()) {
            var ejbReport:InvocationAuditReportVO = (report as InvocationAuditReportVO);
            incrementResource(ejbReport.target, ejbReport.requestDuration, topEjbResName);
        }
        if(report.isWEBInvocation()) {
            var webReport:WebInvocationAuditReportVO = (report as WebInvocationAuditReportVO);
            incrementResource(webReport.target + "/" + webReport.businessMethod, webReport.requestDuration, topWebResName);
        }
        if(report.isJNDIInvocation()) {
            var jndiReport:JNDIAuditReportVO = (report as JNDIAuditReportVO);
            incrementResource(jndiReport.target, jndiReport.requestDuration, topJndiResName);
        }
    }

    public function updateTops():void {
        updateTopsEjb();
        updateTopsWeb();
        updateTopsJndi();
    }

    private function updateTopsEjb():void {
        topEJB.removeAll();
        for (var j:int = 0; j < topLimit; j++) {
            if (j <= (topEjbResName.length - 1)) {
                var name:String =
                        topEjbResName.getItemAt(j).name.split("/")[topEjbResName.getItemAt(j).name.split("/").length -2] 
                        + "/" + topEjbResName.getItemAt(j).name.split("/")[topEjbResName.getItemAt(j).name.split("/").length -1];
                topEJB.addItemAt({name:name, count:topEjbResName.getItemAt(j).count}, 0);
            }
        }
    }

    private function updateTopsWeb():void {
        topWEB.removeAll();
        for (var j:int = 0; j < topLimit; j++) {
            if (j <= (topWebResName.length - 1)) {
                var name:String =
                        topWebResName.getItemAt(j).name.split("/")[topWebResName.getItemAt(j).name.split("/").length - 2] + "/"
                        + topWebResName.getItemAt(j).name.split("/")[topWebResName.getItemAt(j).name.split("/").length - 1];
                topWEB.addItemAt({name:name, count:topWebResName.getItemAt(j).count}, 0);
            }
        }
    }

    private function updateTopsJndi():void {
        topJNDI.removeAll();
        for (var j:int = 0; j < topLimit; j++) {
            if (j <= (topJndiResName.length - 1)) {
                var name:String = topJndiResName.getItemAt(j).name.split(".")[topJndiResName.getItemAt(j).name.split(".").length -1];
                topJNDI.addItemAt({name:name, count:topJndiResName.getItemAt(j).count}, 0);
            }
        }
    }
}
}