/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: WebInvocationAuditReportVO.as 19356 2010-03-04 21:02:36Z mancelin $
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.admin.auditconsole.vo {
import com.adobe.cairngorm.vo.IValueObject;

import mx.collections.ArrayCollection;

import org.ow2.jonas.admin.auditconsole.util.IconsUtils;

/**
 * Object that represent a web audit report.
 */
[RemoteClass(alias="org.ow2.util.auditreport.impl.WebInvocationAuditReport")]
[Bindable]
public class WebInvocationAuditReportVO  extends GenericAuditReportVO implements IValueObject  {

    public var requestStart:Number = 0;

    public var requestStop :Number = 0;

    public var requestDuration:Number = 0;

    public var businessMethod:String = "";

    public var target:String = "";

    [ArrayElementType(Object)]
    public var methodParameters : ArrayCollection = null;

    public var methodReturn:Object = null;

    public var paramSize:Number = 0;

    public var returnSize:Number = 0;

    public var authType:String = "";

    public var contextPath:String = "";

    public var pathInfo:String = "";

    public var pathTranslated:String = "";

    public var queryString:String = "";

    public var requestedSessionId:String = "";

    public var requestURI:String = "";

    public var requestURL:String = "";

    public var servletPath:String = "";

    public var characterEncoding:String = "";

    public var contentLength:int = 0;

    public var contentType:String = "";

    public var localAddress:String = "";

    public var localName:String = "";

    public var localPort:int = 0;

    public var protocol:String = "";

    public var remoteAddress:String = "";

    public var remoteHost:String = "";

    public var remotePort:String = "";

    public var scheme:String = "";

    public var serverName:String = "";

    public var serverport:Number = 0;

	public var status:Number = -1;

    public var secure:Boolean = false;

    public var httpStatusCode:Number = 0;
    /**
     * Constructor.
     */
    public function WebInvocationAuditReportVO() {
        super();
    }

    public override function toString():String {
        var ret:String = super.toString() +
        " Target : " + target +
        " Business method : " + businessMethod +
        " Parameters size : " + paramSize + " bytes" +
		" Status : " + status +
        " Parameters : " + methodParameters +
        " Return : " + methodReturn +
        " Duration : " + requestDuration + " ms" +
        " HTTP status code : " + httpStatusCode +
        " Authentification type : " + authType +
        " Context path : " + contextPath +
        " Translated path : " + pathTranslated +
        " Query : " + queryString +
        " Session ID = " + requestedSessionId +
        " URI : " + requestURI +
        " URL : " + requestURL +
        " Servlet path : " + servletPath +
        " Encoding : " + characterEncoding +
        " Content length : " + contentLength +
        " Content type : " + contentType +
        " Local address : " + localAddress +
        " Local port : " + localPort +
        " Local Name : " + localName +
        " Protocol : " + protocol +
        " Remote address : " + remoteAddress +
        " Remote host : " + remoteHost +
        " Remote port : " + remotePort +
        " Request scheme : " + scheme +
        " Server name : " + serverName +
        " Server port : " + serverport +
        " Stacktrace : " + methodStackTrace;
        return ret
    }

    public override function toHTMLString():String {
        var ret:String = super.toHTMLString() +
        "<font color=\"#FF0000\"><b>Target</b></font> : " + target + "<br/>" +
        "<font color=\"#FF0000\"><b>Business method</b></font> : " + businessMethod + "<br/>" +
		"<font color=\"#FF0000\"><b>Parameters size</b></font> : " + paramSize + " bytes<br/>" +
		"<font color=\"#FF0000\"><b>Status</b></font> : " + status + " <br/>" +
        "<font color=\"#FF0000\"><b>Parameters</b></font> : " + methodParameters + "<br/>" +
        "<font color=\"#FF0000\"><b>Return</b></font> : " + methodReturn + "<br/>" +
        "<font color=\"#FF0000\"><b>Duration</b></font> : " + requestDuration + " ms<br/>" +
        "<font color=\"#FF0000\"><b>HTTP status code</b></font> : " + httpStatusCode + " <br/>" +
        "<font color=\"#FF0000\"><b>Authentification type</b></font> : " + authType + " <br/>" +
        "<font color=\"#FF0000\"><b>Context path</b></font> : " + contextPath + " <br/>" +
        "<font color=\"#FF0000\"><b>Translated path</b></font> : " + pathTranslated + " <br/>" +
        "<font color=\"#FF0000\"><b>Query</b></font> : " + queryString + " <br/>" +
        "<font color=\"#FF0000\"><b>Session ID</b></font> = " + requestedSessionId + " <br/>" +
        "<font color=\"#FF0000\"><b>URI</b></font> : " + requestURI + " <br/>" +
        "<font color=\"#FF0000\"><b>URL</b></font> : " + requestURL + " <br/>" +
        "<font color=\"#FF0000\"><b>Servlet path</b></font> : " + servletPath + " <br/>" +
        "<font color=\"#FF0000\"><b>Encoding</b></font> : " + characterEncoding + " <br/>" +
        "<font color=\"#FF0000\"><b>Content length</b></font> : " + contentLength + " <br/>" +
        "<font color=\"#FF0000\"><b>Content type</b></font> : " + contentType + " <br/>" +
        "<font color=\"#FF0000\"><b>Local address</b></font> : " + localAddress + " <br/>" +
        "<font color=\"#FF0000\"><b>Local port</b></font> : " + localPort + " <br/>" +
        "<font color=\"#FF0000\"><b>Local Name</b></font> : " + localName + " <br/>" +
        "<font color=\"#FF0000\"><b>Protocol</b></font> : " + protocol + " <br/>" +
        "<font color=\"#FF0000\"><b>Remote address</b></font> : " + remoteAddress + " <br/>" +
        "<font color=\"#FF0000\"><b>Remote host</b></font> : " + remoteHost + " <br/>" +
        "<font color=\"#FF0000\"><b>Remote port</b></font> : " + remotePort + " <br/>" +
        "<font color=\"#FF0000\"><b>Request scheme</b></font> : " + scheme + " <br/>" +
        "<font color=\"#FF0000\"><b>Server name</b></font> : " + serverName + " <br/>" +
        "<font color=\"#FF0000\"><b>Server port</b></font> : " + serverport + " <br/>"+
        "<font color=\"#FF0000\"><b>Stacktrace</b></font> : " + methodStackTrace + "<br/>";
        return ret;
    }

    public override function isEJBInvocation():Boolean {
        return false;
    }

    public override function isWEBInvocation():Boolean {
        return true;
    }

    public override function isJNDIInvocation():Boolean {
        return false;
    }

    public override function get icon():String {
        return "/../assets/tomcat.png";
    }

    public override function get iconClass():Class {
        return IconsUtils.getInstance().tomcatIcon;
    }

    public override function get type():String {
        return "WEB";
    }

    public override function get duration():Number {
        return requestDuration;
    }

    public override function get gridTarget():String {
        return businessMethod + " : " + target;
    }
}
}